@@ -5,16 +5,55 @@
 
 // A render_handler that uses SDL & OpenGL
 
+/* $Id: render_handler_ogl.cpp,v 1.74 2007/06/02 09:54:20 tgc Exp $ */
 
-#include "gnash.h"
-#include "types.h"
-#include "image.h"
-#include "ogl.h"
-#include "utility.h"
-
-#include "log.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <cstring>
+#include <cmath>
+
+//#include "gnash.h"
+#include "render_handler.h"
+#include "render_handler_tri.h"
+#include "types.h"
+#include "image.h"
+#include "utility.h"
+
+#if defined(_WIN32) || defined(WIN32)
+#  include <Windows.h>
+#endif
+
+#if defined(NOT_SGI_GL) || defined(__APPLE_CC__)
+#include <AGL/agl.h>
+#include <OpenGL/gl.h>
+#include <OpenGL/glu.h>
+#include <OpenGL/glext.h>
+#else
+# include <GL/gl.h>
+# ifdef WIN32
+#  define GL_CLAMP_TO_EDGE 0x812F
+# else
+# include <GL/glx.h>
+# endif
+# include <GL/glu.h>
+# ifndef APIENTRY
+#  define APIENTRY
+# endif
+#endif
+
+/*
+#if !defined(WIN32) || defined(GUI_GTK)
+# define GL_GLEXT_PROTOTYPES
+# include <GL/gl.h>
+# include <GL/glx.h>
+# include <GL/glu.h>
+# ifndef APIENTRY
+#  define APIENTRY
+# endif
+#endif // no WIN32 or OSX
+*/
 
 using namespace gnash;
 
@@ -25,32 +64,30 @@ using namespace gnash;
 #define RESAMPLE_METHOD 2
 
 
-// Determines whether to generate mipmaps for smoother rendering of
-// minified textures.  It actually tends to look OK to not use
-// mipmaps, though it is potentially a performance hit if you use big
-// textures.
-//
-// TODO: need to code mipmap LOD bias adjustment, to keep mipmaps from
-// looking too blurry.  (Also applies to text rendering.)
-#define GENERATE_MIPMAPS 0
-
 // bitmap_info_ogl declaration
-struct bitmap_info_ogl : public gnash::bitmap_info
+class bitmap_info_ogl : public gnash::bitmap_info
 {
+public:
     bitmap_info_ogl();
-    bitmap_info_ogl(int width, int height, Uint8* data);
+    bitmap_info_ogl(int width, int height, uint8_t* data);
     bitmap_info_ogl(image::rgb* im);
     bitmap_info_ogl(image::rgba* im);
 
-    ~bitmap_info_ogl() {
-	if (m_texture_id > 0) {
-	    glDeleteTextures(1, (GLuint*) &m_texture_id);
+	~bitmap_info_ogl() {
+		if (m_texture_id > 0) {
+			glDeleteTextures(1, (GLuint*) &m_texture_id);
+		}
 	}
-    }
+
+	virtual void layout_image(image::image_base* im);
 };
 
-struct render_handler_ogl : public gnash::render_handler
+// static GLint iquad[] = {-1, 1, 1, 1, 1, -1, -1, -1};
+
+class render_handler_ogl : public gnash::triangulating_render_handler
 {
+public:
+
     // Some renderer state.
     
     // Enable/disable antialiasing.
@@ -68,7 +105,7 @@ struct render_handler_ogl : public gnash::render_handler
 
     // Utility.  Mutates *width, *height and *data to create the
     // next mip level.
-    static void make_next_miplevel(int* width, int* height, Uint8* data)
+    static void make_next_miplevel(int* width, int* height, uint8_t* data)
 	{
 	    assert(width);
 	    assert(height);
@@ -89,8 +126,8 @@ struct render_handler_ogl : public gnash::render_handler
 	    } else {
 		// Resample.  Simple average 2x2 --> 1, in-place.
 		for (int j = 0; j < new_h; j++) {
-		    Uint8*	out = ((Uint8*) data) + j * new_w;
-		    Uint8*	in = ((Uint8*) data) + (j << 1) * *width;
+		    uint8_t*	out = ((uint8_t*) data) + j * new_w;
+		    uint8_t*	in = ((uint8_t*) data) + (j << 1) * *width;
 		    for (int i = 0; i < new_w; i++) {
 			int	a;
 			a = (*(in + 0) + *(in + 1) + *(in + 0 + *width) + *(in + 1 + *width));
@@ -106,8 +143,9 @@ struct render_handler_ogl : public gnash::render_handler
 	    *height = new_h;
 	}
     
-    struct fill_style
+    class fill_style
     {
+    public:
 	enum mode
 	{
 	    INVALID,
@@ -115,11 +153,11 @@ struct render_handler_ogl : public gnash::render_handler
 	    BITMAP_WRAP,
 	    BITMAP_CLAMP,
 	    LINEAR_GRADIENT,
-	    RADIAL_GRADIENT,
+	    RADIAL_GRADIENT
 	};
 	mode	m_mode;
 	gnash::rgba	m_color;
-	const gnash::bitmap_info*	m_bitmap_info;
+	gnash::bitmap_info*	m_bitmap_info;
 	gnash::matrix	m_bitmap_matrix;
 	gnash::cxform	m_bitmap_color_transform;
 	bool	m_has_nonzero_bitmap_additive_color;
@@ -162,7 +200,16 @@ struct render_handler_ogl : public gnash::render_handler
 			    );
 		    }
 		    
-		    glBindTexture(GL_TEXTURE_2D, m_bitmap_info->m_texture_id);
+				if (m_bitmap_info->m_texture_id == 0 && m_bitmap_info->m_suspended_image != NULL)
+				{
+					m_bitmap_info->layout_image(m_bitmap_info->m_suspended_image);
+					delete m_bitmap_info->m_suspended_image;
+					m_bitmap_info->m_suspended_image = NULL;
+				}
+
+				// assert(m_bitmap_info->m_texture_id);
+
+				glBindTexture(GL_TEXTURE_2D, m_bitmap_info->m_texture_id);
 		    glEnable(GL_TEXTURE_2D);
 		    glEnable(GL_TEXTURE_GEN_S);
 		    glEnable(GL_TEXTURE_GEN_T);
@@ -245,16 +292,16 @@ struct render_handler_ogl : public gnash::render_handler
 	void	set_bitmap(const gnash::bitmap_info* bi, const gnash::matrix& m, bitmap_wrap_mode wm, const gnash::cxform& color_transform)
 	    {
 		m_mode = (wm == WRAP_REPEAT) ? BITMAP_WRAP : BITMAP_CLAMP;
-		m_bitmap_info = bi;
+		m_bitmap_info = const_cast<gnash::bitmap_info*> ( bi );
 		m_bitmap_matrix = m;
 		m_bitmap_color_transform = color_transform;
 		m_bitmap_color_transform.clamp();
 			
 		m_color = gnash::rgba(
-		    Uint8(m_bitmap_color_transform.m_[0][0] * 255.0f),
-		    Uint8(m_bitmap_color_transform.m_[1][0] * 255.0f),
-		    Uint8(m_bitmap_color_transform.m_[2][0] * 255.0f),
-		    Uint8(m_bitmap_color_transform.m_[3][0] * 255.0f));
+		    uint8_t(m_bitmap_color_transform.m_[0][0] * 255.0f),
+		    uint8_t(m_bitmap_color_transform.m_[1][0] * 255.0f),
+		    uint8_t(m_bitmap_color_transform.m_[2][0] * 255.0f),
+		    uint8_t(m_bitmap_color_transform.m_[3][0] * 255.0f));
 			
 		if (m_bitmap_color_transform.m_[0][1] > 1.0f
 		    || m_bitmap_color_transform.m_[1][1] > 1.0f
@@ -285,7 +332,7 @@ struct render_handler_ogl : public gnash::render_handler
 
 
     gnash::bitmap_info*	create_bitmap_info_rgb(image::rgb* im)
-	// Given an image, returns a pointer to a bitmap_info struct
+	// Given an image, returns a pointer to a bitmap_info class
 	// that can later be passed to fill_styleX_bitmap(), to set a
 	// bitmap fill style.
 	{
@@ -294,7 +341,7 @@ struct render_handler_ogl : public gnash::render_handler
 
 
     gnash::bitmap_info*	create_bitmap_info_rgba(image::rgba* im)
-	// Given an image, returns a pointer to a bitmap_info struct
+	// Given an image, returns a pointer to a bitmap_info class
 	// that can later be passed to fill_style_bitmap(), to set a
 	// bitmap fill style.
 	//
@@ -303,18 +350,7 @@ struct render_handler_ogl : public gnash::render_handler
 	    return new bitmap_info_ogl(im);
 	}
 
-
-    gnash::bitmap_info*	create_bitmap_info_empty()
-	// Create a placeholder bitmap_info.  Used when
-	// DO_NOT_LOAD_BITMAPS is set; then later on the host program
-	// can use movie_definition::get_bitmap_info_count() and
-	// movie_definition::get_bitmap_info() to stuff precomputed
-	// textures into these bitmap infos.
-	{
-	    return new bitmap_info_ogl;
-	}
-
-    gnash::bitmap_info*	create_bitmap_info_alpha(int w, int h, Uint8* data)
+    gnash::bitmap_info*	create_bitmap_info_alpha(int w, int h, uint8_t* data)
 	// Create a bitmap_info so that it contains an alpha texture
 	// with the given data (1 byte per texel).
 	//
@@ -325,16 +361,109 @@ struct render_handler_ogl : public gnash::render_handler
 
 
     void	delete_bitmap_info(gnash::bitmap_info* bi)
-	// Delete the given bitmap info struct.
+	// Delete the given bitmap info class.
 	{
 	    delete bi;
 	}
 
+#define GLYUV 0
 
-    ~render_handler_ogl()
+ 	// Returns the format the current renderer wants videoframes in.
+	int videoFrameFormat() {
+#if GLYUV
+		return YUV;
+#else
+		return RGB;
+#endif
+	}
+	
+	/// Draws the video frames
+	void drawVideoFrame(image::image_base* baseframe, const matrix* m, const rect* bounds){
+#if GLYUV
+		image::yuv* frame = static_cast<image::yuv*>(baseframe);
+#else
+		image::rgb* frame = static_cast<image::rgb*>(baseframe);
+#endif
+		glPushAttrib(GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT);
+
+#if GLYUV
+		static GLfloat yuv_rgb[16] = {
+			1, 1, 1, 0,
+			0, -0.3946517043589703515f, 2.032110091743119266f, 0,
+			1.139837398373983740f, -0.5805986066674976801f, 0, 0,
+			0, 0, 0, 1
+		};
+#endif
+
+		glMatrixMode(GL_COLOR);
+		glPushMatrix();
+#if GLYUV
+		glLoadMatrixf(yuv_rgb);
+		glPixelTransferf(GL_GREEN_BIAS, -0.5f);
+		glPixelTransferf(GL_BLUE_BIAS, -0.5f);
+#else
+		glLoadIdentity();
+		glPixelTransferf(GL_GREEN_BIAS, 0.0);
+		glPixelTransferf(GL_BLUE_BIAS, 0.0);
+#endif
+		gnash::point a, b, c, d;
+		m->transform(&a, gnash::point(bounds->get_x_min(), bounds->get_y_min()));
+		m->transform(&b, gnash::point(bounds->get_x_max(), bounds->get_y_min()));
+		m->transform(&c, gnash::point(bounds->get_x_min(), bounds->get_y_max()));
+		d.m_x = b.m_x + c.m_x - a.m_x;
+		d.m_y = b.m_y + c.m_y - a.m_y;
+
+		float w_bounds = TWIPS_TO_PIXELS(b.m_x - a.m_x);
+		float h_bounds = TWIPS_TO_PIXELS(c.m_y - a.m_y);
+
+		unsigned char*   ptr = frame->m_data;
+		float xpos = a.m_x < 0 ? 0.0f : a.m_x;	//hack
+		float ypos = a.m_y < 0 ? 0.0f : a.m_y;	//hack
+		glRasterPos2f(xpos, ypos);	//hack
+
+#if GLYUV
+		GLenum rgb[3] = {GL_RED, GL_GREEN, GL_BLUE}; 
+
+		for (int i = 0; i < 3; ++i)
+		{
+			float zx = w_bounds / (float) frame->planes[i].w;
+			float zy = h_bounds / (float) frame->planes[i].h;
+			glPixelZoom(zx, - zy);	// flip & zoom image
+
+			if (i > 0)
+			{
+				glEnable(GL_BLEND);
+				glBlendFunc(GL_ONE, GL_ONE);
+			}
+
+			glDrawPixels(frame->planes[i].w, frame->planes[i].h, rgb[i], GL_UNSIGNED_BYTE, ptr);
+			ptr += frame->planes[i].size;
+		}
+#else
+		int height = frame->m_height;
+		int width = frame->m_width;
+		float zx = w_bounds / (float) width;
+		float zy = h_bounds / (float) height;
+		glPixelZoom(zx,  -zy);	// flip & zoom image
+		glDrawPixels(width, height, GL_RGB, GL_UNSIGNED_BYTE, ptr);
+#endif
+
+		glMatrixMode(GL_COLOR);
+		glPopMatrix();
+
+		glPopAttrib();
+	
+	}
+   
+	// Ctor stub.
+	render_handler_ogl()
 	{
 	}
 
+	// Dtor stub.
+	~render_handler_ogl()
+	{
+	}
 
     void	begin_display(
 	gnash::rgba background_color,
@@ -385,48 +514,65 @@ struct render_handler_ogl : public gnash::render_handler
 		    glEnd();
 		}
 
-	    // Old unused code.  Might get revived someday.
-// #if 0
-// 	    // See if we want to, and can, use multitexture
-// 	    // antialiasing.
-// 	    s_multitexture_antialias = false;
-// 	    if (m_enable_antialias)
-// 		{
-// 		    int	tex_units = 0;
-// 		    glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &tex_units);
-// 		    if (tex_units >= 2)
-// 			{
-// 			    s_multitexture_antialias = true;
-// 			}
+	    // Markus: Implement anti-aliasing here...
+#if 0
+/*
+Code from Timo Kanera...
+if (gl_antialias)
+  {
+    glShadeModel (GL_SMOOTH);
+    glEnable (GL_POLYGON_SMOOTH);
+    glEnable (GL_LINE_SMOOTH);
+    glEnable (GL_POINT_SMOOTH);
+    mp_msg(MSGT_VO, MSGL_INFO, "[sgi] antialiasing on\n");
+}
+else {
+    glShadeModel (GL_FLAT);
+    glDisable (GL_POLYGON_SMOOTH);
+    glDisable (GL_LINE_SMOOTH);
+    glDisable (GL_POINT_SMOOTH);
+    mp_msg(MSGT_VO, MSGL_INFO, "[sgi] antialiasing off\n");
+}
+*/
+ 	    // See if we want to, and can, use multitexture
+ 	    // antialiasing.
+ 	    bool s_multitexture_antialias = false;
+ 	    if (m_enable_antialias)
+ 		{
+ 		    int	tex_units = 0;
+ 		    glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &tex_units);
+ 		    if (tex_units >= 2)
+ 			{
+ 			    s_multitexture_antialias = true;
+ 			}
+ 		    // Make sure we have an edge texture available.
+ 		    if (s_multitexture_antialias == true)
+ 			{
+ 			    // Very simple texture: 2 texels wide, 1 texel high.
+ 			    // Both texels are white; left texel is all clear, right texel is all opaque.
+ 			    unsigned char	edge_data[8] = { 255, 255, 255, 0, 255, 255, 255, 255 };
+			    GLuint s_edge_texture_id = 0;
 
-// 		    // Make sure we have an edge texture available.
-// 		    if (s_multitexture_antialias == true
-// 			&& s_edge_texture_id == 0)
-// 			{
-// 			    // Very simple texture: 2 texels wide, 1 texel high.
-// 			    // Both texels are white; left texel is all clear, right texel is all opaque.
-// 			    unsigned char	edge_data[8] = { 255, 255, 255, 0, 255, 255, 255, 255 };
+ 			    glActiveTextureARB(GL_TEXTURE1_ARB);
+ 			    glEnable(GL_TEXTURE_2D);
+ 			    glGenTextures(1, (GLuint*)&s_edge_texture_id);
+ 			    glBindTexture(GL_TEXTURE_2D, s_edge_texture_id);
 
-// 			    ogl::active_texture(GL_TEXTURE1_ARB);
-// 			    glEnable(GL_TEXTURE_2D);
-// 			    glGenTextures(1, &s_edge_texture_id);
-// 			    glBindTexture(GL_TEXTURE_2D, s_edge_texture_id);
+ 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+ 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+ 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+ 			    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 1, 0, GL_RGBA, GL_UNSIGNED_BYTE, edge_data);
 
-// 			    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 1, 0, GL_RGBA, GL_UNSIGNED_BYTE, edge_data);
+ 			    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);	// @@ should we use a 1D texture???
 
-// 			    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);	// @@ should we use a 1D texture???
-
-// 			    glDisable(GL_TEXTURE_2D);
-// 			    ogl::active_texture(GL_TEXTURE0_ARB);
-// 			    glDisable(GL_TEXTURE_2D);
-// 			}
-// 		}
-// #endif // 0
+ 			    glDisable(GL_TEXTURE_2D);
+ 			    glActiveTextureARB(GL_TEXTURE0_ARB);
+ 			    glDisable(GL_TEXTURE_2D);
+ 			}
+ 		}
+#endif // 0
 	}
 
 
@@ -518,9 +664,18 @@ struct render_handler_ogl : public gnash::render_handler
 	
     void	line_style_width(float width)
 	{
-	    // TODO: OpenGL doesn't seem to handle very low-width lines well, even with anti-aliasing enabled
-	    // But this is a start (20 TWIPS' width = 1 pixel's)
-	    glLineWidth(TWIPS_TO_PIXELS(width));
+		if ( width == 1.0 ) // "hairline", see render_handler_tri.h
+		{
+			glLineWidth(1); // expected: 1 pixel
+		}
+		else
+		{
+			// TODO: OpenGL doesn't seem to handle very
+			// low-width lines well, even with anti-aliasing
+			// enabled
+			// But this is a start (20 TWIPS' width = 1 pixel's)
+			glLineWidth(TWIPS_TO_PIXELS(width));
+		}
 	}
 
 
@@ -541,7 +696,7 @@ struct render_handler_ogl : public gnash::render_handler
 
 	    // Send the tris to OpenGL
 	    glEnableClientState(GL_VERTEX_ARRAY);
-	    glVertexPointer(2, GL_SHORT, sizeof(Sint16) * 2, coords);
+	    glVertexPointer(2, GL_SHORT, sizeof(int16_t) * 2, coords);
 	    glDrawArrays(GL_TRIANGLE_STRIP, 0, vertex_count);
 
 	    if (m_current_styles[LEFT_STYLE].needs_second_pass())
@@ -607,7 +762,7 @@ struct render_handler_ogl : public gnash::render_handler
 	    // antialiased alpha mask of the mesh shape, in the
 	    // destination alpha channel.
 	    glEnableClientState(GL_VERTEX_ARRAY);
-	    glVertexPointer(2, GL_SHORT, sizeof(Sint16) * 2, coords);
+	    glVertexPointer(2, GL_SHORT, sizeof(int16_t) * 2, coords);
 	    glDrawArrays(GL_TRIANGLE_STRIP, 0, vertex_count);
 	    glDisableClientState(GL_VERTEX_ARRAY);
 
@@ -655,7 +810,7 @@ struct render_handler_ogl : public gnash::render_handler
 
 	    // Send the line-strip to OpenGL
 	    glEnableClientState(GL_VERTEX_ARRAY);
-	    glVertexPointer(2, GL_SHORT, sizeof(Sint16) * 2, coords);
+	    glVertexPointer(2, GL_SHORT, sizeof(int16_t) * 2, coords);
 	    glDrawArrays(GL_LINE_STRIP, 0, vertex_count);
 	    glDisableClientState(GL_VERTEX_ARRAY);
 
@@ -681,29 +836,43 @@ struct render_handler_ogl : public gnash::render_handler
 	    apply_color(color);
 
 	    gnash::point a, b, c, d;
-	    m.transform(&a, gnash::point(coords.m_x_min, coords.m_y_min));
-	    m.transform(&b, gnash::point(coords.m_x_max, coords.m_y_min));
-	    m.transform(&c, gnash::point(coords.m_x_min, coords.m_y_max));
+	    m.transform(&a, gnash::point(coords.get_x_min(), coords.get_y_min()));
+	    m.transform(&b, gnash::point(coords.get_x_max(), coords.get_y_min()));
+	    m.transform(&c, gnash::point(coords.get_x_min(), coords.get_y_max()));
 	    d.m_x = b.m_x + c.m_x - a.m_x;
 	    d.m_y = b.m_y + c.m_y - a.m_y;
 
-	    glBindTexture(GL_TEXTURE_2D, bi->m_texture_id);
+			if (bi->m_texture_id == 0 && bi->m_suspended_image != NULL)
+			{
+				const_cast<bitmap_info*>(bi)->layout_image(bi->m_suspended_image);
+				delete bi->m_suspended_image;
+				const_cast<bitmap_info*>(bi)->m_suspended_image = NULL;
+			}
+
+			// assert(bi->m_texture_id);
+
+			glBindTexture(GL_TEXTURE_2D, bi->m_texture_id);
 	    glEnable(GL_TEXTURE_2D);
 	    glDisable(GL_TEXTURE_GEN_S);
 	    glDisable(GL_TEXTURE_GEN_T);
 
 	    glBegin(GL_TRIANGLE_STRIP);
 
-	    glTexCoord2f(uv_coords.m_x_min, uv_coords.m_y_min);
+            float xmin = uv_coords.get_x_min();
+            float xmax = uv_coords.get_x_max();
+            float ymin = uv_coords.get_y_min();
+            float ymax = uv_coords.get_y_max();
+
+	    glTexCoord2f(xmin, ymin);
 	    glVertex2f(a.m_x, a.m_y);
 
-	    glTexCoord2f(uv_coords.m_x_max, uv_coords.m_y_min);
+	    glTexCoord2f(xmax, ymin);
 	    glVertex2f(b.m_x, b.m_y);
 
-	    glTexCoord2f(uv_coords.m_x_min, uv_coords.m_y_max);
+	    glTexCoord2f(xmin, ymax);
 	    glVertex2f(c.m_x, c.m_y);
 
-	    glTexCoord2f(uv_coords.m_x_max, uv_coords.m_y_max);
+	    glTexCoord2f(xmax, ymax);
 	    glVertex2f(d.m_x, d.m_y);
 
 	    glEnd();
@@ -735,14 +904,19 @@ struct render_handler_ogl : public gnash::render_handler
 	    glDisable(GL_STENCIL_TEST); 
 	}
 	
-};	// end struct render_handler_ogl
+};	// end class render_handler_ogl
 
 
 // bitmap_info_ogl implementation
 
+/*
 
+Markus: A. A. I still miss you and the easter 2006, you know.
+	A. J. I miss you too, but you'll probably not read this code ever... :/
 
-void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8* src_data, int dst_width, int dst_height)
+*/
+
+void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8_t* src_data, int dst_width, int dst_height)
 // Code from Alex Streit
 //
 // Sets the current texture to a resampled/expanded version of the
@@ -785,7 +959,7 @@ void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8
 	}
 	glEnd();
 	glCopyTexImage2D(GL_TEXTURE_2D, 0, out_format, 0,0, dst_width, dst_height, 0);
-	delete temp;
+	delete[] temp;
     }
     glPopAttrib();
     glPopMatrix();
@@ -793,36 +967,13 @@ void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8
 }
 
 
-void	generate_mipmaps(unsigned int internal_format, unsigned int input_format, int bytes_per_pixel, image::image_base* im)
-// DESTRUCTIVELY generate mipmaps of the given image.  The image data
-// and width/height of im are munged in this process.
-{
-//    GNASH_REPORT_FUNCTION;
-    int	level = 1;
-    while (im->m_width > 1 || im->m_height > 1)
-	{
-	    if (bytes_per_pixel == 3)
-		{
-		    image::make_next_miplevel((image::rgb*) im);
-		}
-	    else
-		{
-		    image::make_next_miplevel((image::rgba*) im);
-		}
-
-	    glTexImage2D(GL_TEXTURE_2D, level, internal_format, im->m_width, im->m_height, 0,
-			 input_format, GL_UNSIGNED_BYTE, im->m_data);
-	    level++;
-	}
-}
-
 
 void	software_resample(
     int bytes_per_pixel,
     int src_width,
     int src_height,
     int src_pitch,
-    uint8* src_data,
+    uint8_t* src_data,
     int dst_width,
     int dst_height)
 // Code from Alex Streit
@@ -842,12 +993,12 @@ void	software_resample(
 
     // FAST bi-linear filtering
     // the code here is designed to be fast, not readable
-    Uint8* rescaled = new Uint8[dst_width * dst_height * bytes_per_pixel];
+    uint8_t* rescaled = new uint8_t[dst_width * dst_height * bytes_per_pixel];
     float Uf, Vf;		// fractional parts
     float Ui, Vi;		// integral parts
     float w1, w2, w3, w4;	// weighting
-    Uint8* psrc;
-    Uint8* pdst = rescaled;
+    uint8_t* psrc;
+    uint8_t* pdst = rescaled;
     // i1,i2,i3,i4 are the offsets of the surrounding 4 pixels
     const int i1 = 0;
     const int i2 = bytes_per_pixel;
@@ -861,7 +1012,7 @@ void	software_resample(
     float V=0;
 
 #define BYTE_SAMPLE(offset)	\
-	(Uint8) (w1 * psrc[i1 + (offset)] + w2 * psrc[i2 + (offset)] + w3 * psrc[i3 + (offset)] + w4 * psrc[i4 + (offset)])
+	(uint8_t) (w1 * psrc[i1 + (offset)] + w2 * psrc[i2 + (offset)] + w3 * psrc[i3 + (offset)] + w4 * psrc[i4 + (offset)])
 
     if (bytes_per_pixel == 3)
 	{
@@ -889,19 +1040,6 @@ void	software_resample(
 			    psrc += 3;
 			}
 		}
-
-#ifdef DEBUG_WRITE_TEXTURES_TO_PPM
-	    static int s_image_sequence = 0;
-	    char temp[256];
-	    sprintf(temp, "image%d.ppm", s_image_sequence++);
-	    FILE* f = fopen(temp, "wb");
-	    if (f)
-		{
-		    fprintf(f, "P6\n# test code\n%d %d\n255\n", dst_width, dst_height);
-		    fwrite(rescaled, dst_width * dst_height * 3, 1, f);
-		    fclose(f);
-		}
-#endif
 	}
     else
 	{
@@ -936,12 +1074,6 @@ void	software_resample(
 
     glTexImage2D(GL_TEXTURE_2D, 0, internal_format, dst_width, dst_height, 0, input_format, GL_UNSIGNED_BYTE, rescaled);
 
-#if GENERATE_MIPMAPS
-    // Build mipmaps.
-    image::image_base	im(rescaled, dst_width, dst_height, dst_width * bytes_per_pixel);
-    generate_mipmaps(internal_format, input_format, bytes_per_pixel, &im);
-#endif // GENERATE_MIPMAPS
-
     delete [] rescaled;
 }
 
@@ -956,52 +1088,198 @@ bitmap_info_ogl::bitmap_info_ogl()
     m_original_height = 0;
 }
 
+void bitmap_info_ogl::layout_image(image::image_base* im)
+{
 
-bitmap_info_ogl::bitmap_info_ogl(int width, int height, Uint8* data)
+	assert(im);
+
+	// Create the texture.
+	glEnable(GL_TEXTURE_2D);
+	glGenTextures(1, (GLuint*)&m_texture_id);
+	glBindTexture(GL_TEXTURE_2D, m_texture_id);
+
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+
+	m_original_width = im->m_width;
+	m_original_height = im->m_height;
+
+	switch (im->m_type)
+	{
+		case image::image_base::RGB:
+		{
+
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+
+			int	w = 1; while (w < im->m_width) { w <<= 1; }
+			int	h = 1; while (h < im->m_height) { h <<= 1; }
+
+			if (w != im->m_width	|| h != im->m_height)
+			{
+#if (RESAMPLE_METHOD == 1)
+				int	viewport_dim[2] = { 0, 0 };
+				glGetIntegerv(GL_MAX_VIEWPORT_DIMS, &viewport_dim[0]);
+				if (w > viewport_dim[0]
+			|| h > viewport_dim[1]
+			|| im->m_width * 3 != im->m_pitch)
+			{
+					// Can't use hardware resample.  Either frame
+					// buffer isn't big enough to fit the source
+					// texture, or the source data isn't padded
+					// quite right.
+					software_resample(3, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
+			}
+				else
+			{
+					hardware_resample(3, im->m_width, im->m_height, im->m_data, w, h);
+			}
+#elif (RESAMPLE_METHOD == 2)
+				{
+			// Faster/simpler software bilinear rescale.
+			software_resample(3, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
+				}
+#else
+				{
+			// Fancy but slow software resampling.
+			image::rgb*	rescaled = image::create_rgb(w, h);
+			image::resample(rescaled, 0, 0, w - 1, h - 1,
+					im, 0, 0, (float) im->m_width, (float) im->m_height);
+
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, rescaled->m_data);
+
+			delete rescaled;
+				}
+#endif
+			}
+			else
+			{
+				// Use original image directly.
+				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, im->m_data);
+			}
+
+			break;
+		}
+		case image::image_base::RGBA:
+		{
+
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+
+			int	w = 1; while (w < im->m_width) { w <<= 1; }
+			int	h = 1; while (h < im->m_height) { h <<= 1; }
+
+			if (w != im->m_width	|| h != im->m_height)
+			{
+#if (RESAMPLE_METHOD == 1)
+				int	viewport_dim[2] = { 0, 0 };
+				glGetIntegerv(GL_MAX_VIEWPORT_DIMS, &viewport_dim[0]);
+				if (w > viewport_dim[0]
+			|| h > viewport_dim[1]
+			|| im->m_width * 4 != im->m_pitch)
+			{
+					// Can't use hardware resample.  Either frame
+					// buffer isn't big enough to fit the source
+					// texture, or the source data isn't padded
+					// quite right.
+					software_resample(4, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
+			}
+				else
+			{
+					hardware_resample(4, im->m_width, im->m_height, im->m_data, w, h);
+			}
+#elif (RESAMPLE_METHOD == 2)
+				{
+			// Faster/simpler software bilinear rescale.
+			software_resample(4, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
+				}
+#else
+				{
+			// Fancy but slow software resampling.
+			image::rgba*	rescaled = image::create_rgba(w, h);
+			image::resample(rescaled, 0, 0, w - 1, h - 1,
+					im, 0, 0, (float) im->m_width, (float) im->m_height);
+
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, rescaled->m_data);
+
+			delete rescaled;
+				}
+#endif
+			}
+			else
+			{
+				// Use original image directly.
+				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, im->m_data);
+			}
+
+			break;
+		}
+		case image::image_base::ROW:
+		{
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+
+#ifndef NDEBUG
+			// You must use power-of-two dimensions!!
+			int	w = 1; while (w < im->m_width) { w <<= 1; }
+			int	h = 1; while (h < im->m_height) { h <<= 1; }
+			assert(w == im->m_width);
+			assert(h == im->m_height);
+#endif // not NDEBUG
+
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA,
+				im->m_width, im->m_height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, im->m_data);
+
+			// Build mips.
+			int	level = 1;
+			while (im->m_width > 1 || im->m_height > 1)
+			{
+				render_handler_ogl::make_next_miplevel(&im->m_width, &im->m_height, im->m_data);
+				glTexImage2D(GL_TEXTURE_2D, level, GL_ALPHA, 
+					im->m_width, im->m_height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, im->m_data);
+				level++;
+			}
+
+			break;
+		}
+
+		default:
+//			printf("unsupported image type\n");
+			break;
+	}
+
+}
+inline bool opengl_accessible()
+{
+#if defined(_WIN32) || defined(WIN32)
+	return wglGetCurrentContext() != 0;
+#elif defined(__APPLE_CC__)
+	return aglGetCurrentContext() != 0;
+#else
+	return glXGetCurrentContext() != 0;
+#endif
+}
+
+bitmap_info_ogl::bitmap_info_ogl(int width, int height, uint8_t* data)
 // Initialize this bitmap_info to an alpha image
 // containing the specified data (1 byte per texel).
 //
 // !! Munges *data in order to create mipmaps !!
 {
 //    GNASH_REPORT_FUNCTION;
-    assert(width > 0);
-    assert(height > 0);
-    assert(data);
+	assert(width > 0);
+	assert(height > 0);
+	assert(data);
 
-    m_texture_id = 0;
-	
-    // Create the texture.
+	// TODO optimization
+	image::image_base* im = new image::image_base(data, width, height, 1, image::image_base::ROW);
+	memcpy(im->m_data, data, width * height);
 
-    glEnable(GL_TEXTURE_2D);
-    glGenTextures(1, (GLuint*)&m_texture_id);
-    glBindTexture(GL_TEXTURE_2D, m_texture_id);
-
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// GL_NEAREST ?
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-
-    m_original_width = width;
-    m_original_height = height;
-
-#ifndef NDEBUG
-    // You must use power-of-two dimensions!!
-    int	w = 1; while (w < width) { w <<= 1; }
-    int	h = 1; while (h < height) { h <<= 1; }
-    assert(w == width);
-    assert(h == height);
-#endif // not NDEBUG
-
-    glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, width, height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, data);
-
-    // Build mips.
-    int	level = 1;
-    while (width > 1 || height > 1)
+	if (opengl_accessible() == false) 
 	{
-	    render_handler_ogl::make_next_miplevel(&width, &height, data);
-	    glTexImage2D(GL_TEXTURE_2D, level, GL_ALPHA, width, height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, data);
-	    level++;
+		m_suspended_image = im;
+		return;
 	}
+	layout_image(im);
+	delete im;
 }
 
 
@@ -1009,78 +1287,15 @@ bitmap_info_ogl::bitmap_info_ogl(image::rgb* im)
 // NOTE: This function destroys im's data in the process of making mipmaps.
 {
 //    GNASH_REPORT_FUNCTION;
-    assert(im);
+	assert(im);
 
-    // Create the texture.
-
-    glEnable(GL_TEXTURE_2D);
-    glGenTextures(1, (GLuint*)&m_texture_id);
-    glBindTexture(GL_TEXTURE_2D, m_texture_id);
-
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-#if GENERATE_MIPMAPS
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-#else
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-#endif
-
-    m_original_width = im->m_width;
-    m_original_height = im->m_height;
-
-    int	w = 1; while (w < im->m_width) { w <<= 1; }
-    int	h = 1; while (h < im->m_height) { h <<= 1; }
-
-    if (w != im->m_width
-	|| h != im->m_height)
+	if (opengl_accessible() == false) 
 	{
-#if (RESAMPLE_METHOD == 1)
-	    int	viewport_dim[2] = { 0, 0 };
-	    glGetIntegerv(GL_MAX_VIEWPORT_DIMS, &viewport_dim[0]);
-	    if (w > viewport_dim[0]
-		|| h > viewport_dim[1]
-		|| im->m_width * 3 != im->m_pitch)
-		{
-		    // Can't use hardware resample.  Either frame
-		    // buffer isn't big enough to fit the source
-		    // texture, or the source data isn't padded
-		    // quite right.
-		    software_resample(3, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
-		}
-	    else
-		{
-		    hardware_resample(3, im->m_width, im->m_height, im->m_data, w, h);
-		}
-#elif (RESAMPLE_METHOD == 2)
-	    {
-		// Faster/simpler software bilinear rescale.
-		software_resample(3, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
-	    }
-#else
-	    {
-		// Fancy but slow software resampling.
-		image::rgb*	rescaled = image::create_rgb(w, h);
-		image::resample(rescaled, 0, 0, w - 1, h - 1,
-				im, 0, 0, (float) im->m_width, (float) im->m_height);
-
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, rescaled->m_data);
-#if GENERATE_MIPMAPS
-		generate_mipmaps(GL_RGB, GL_RGB, 3, rescaled);
-#endif // GENERATE_MIPMAPS
-
-		delete rescaled;
-	    }
-#endif
-	}
-    else
-	{
-	    // Use original image directly.
-	    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, im->m_data);
-#if GENERATE_MIPMAPS
-	    generate_mipmaps(GL_RGB, GL_RGB, 3, im);
-#endif // GENERATE_MIPMAPS
+		m_suspended_image = image::create_rgb(im->m_width, im->m_height);
+		memcpy(m_suspended_image->m_data, im->m_data, im->m_pitch * im->m_height);
+		return;
 	}
+	layout_image(im);
 }
 
 
@@ -1089,78 +1304,15 @@ bitmap_info_ogl::bitmap_info_ogl(image::rgba* im)
 // NOTE: This function destroys im's data in the process of making mipmaps.
 {
 //    GNASH_REPORT_FUNCTION;
-    assert(im);
+	assert(im);
 
-    // Create the texture.
-
-    glEnable(GL_TEXTURE_2D);
-    glGenTextures(1, (GLuint*)&m_texture_id);
-    glBindTexture(GL_TEXTURE_2D, m_texture_id);
-
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// GL_NEAREST ?
-#if GENERATE_MIPMAPS
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-#else
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-#endif
-
-    m_original_width = im->m_width;
-    m_original_height = im->m_height;
-
-    int	w = 1; while (w < im->m_width) { w <<= 1; }
-    int	h = 1; while (h < im->m_height) { h <<= 1; }
-
-    if (w != im->m_width
-	|| h != im->m_height)
+	if (opengl_accessible() == false) 
 	{
-#if (RESAMPLE_METHOD == 1)
-	    int	viewport_dim[2] = { 0, 0 };
-	    glGetIntegerv(GL_MAX_VIEWPORT_DIMS, &viewport_dim[0]);
-	    if (w > viewport_dim[0]
-		|| h > viewport_dim[1]
-		|| im->m_width * 4 != im->m_pitch)
-		{
-		    // Can't use hardware resample.  Either frame
-		    // buffer isn't big enough to fit the source
-		    // texture, or the source data isn't padded
-		    // quite right.
-		    software_resample(4, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
-		}
-	    else
-		{
-		    hardware_resample(4, im->m_width, im->m_height, im->m_data, w, h);
-		}
-#elif (RESAMPLE_METHOD == 2)
-	    {
-		// Faster/simpler software bilinear rescale.
-		software_resample(4, im->m_width, im->m_height, im->m_pitch, im->m_data, w, h);
-	    }
-#else
-	    {
-		// Fancy but slow software resampling.
-		image::rgba*	rescaled = image::create_rgba(w, h);
-		image::resample(rescaled, 0, 0, w - 1, h - 1,
-				im, 0, 0, (float) im->m_width, (float) im->m_height);
-
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, rescaled->m_data);
-#if GENERATE_MIPMAPS
-		generate_mipmaps(GL_RGBA, GL_RGBA, 4, rescaled);
-#endif // GENERATE_MIPMAPS
-
-		delete rescaled;
-	    }
-#endif
-	}
-    else
-	{
-	    // Use original image directly.
-	    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, im->m_data);
-#if GENERATE_MIPMAPS
-	    generate_mipmaps(GL_RGBA, GL_RGBA, 4, im);
-#endif // GENERATE_MIPMAPS
+		m_suspended_image = image::create_rgba(im->m_width, im->m_height);
+		memcpy(m_suspended_image->m_data, im->m_data, im->m_pitch * im->m_height);
+		return;
 	}
+	layout_image(im);
 }
 
 
@@ -1168,6 +1320,51 @@ gnash::render_handler*	gnash::create_render_handler_ogl()
 // Factory.
 {
 //    GNASH_REPORT_FUNCTION;
+
+    // Do some initialisation.
+#define OVERSIZE	1.0f
+    
+    //This makes fonts look nice (actually!)
+#if 0
+    glEnable(GL_POLYGON_SMOOTH);
+    glEnable(GL_POINT_SMOOTH);
+#endif    
+        // Turn on alpha blending.
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+;        
+        // Turn on line smoothing.  Antialiased lines can be used to
+        // smooth the outsides of shapes.
+        glEnable(GL_LINE_SMOOTH);
+        glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);	// GL_NICEST, GL_FASTEST, GL_DONT_CARE
+        
+        glMatrixMode(GL_PROJECTION);
+        glOrtho(-OVERSIZE, OVERSIZE, OVERSIZE, -OVERSIZE, -1, 1);
+        glMatrixMode(GL_MODELVIEW);
+        glLoadIdentity();
+        
+        // We don't need lighting effects
+        glDisable(GL_LIGHTING);
+        // glColorPointer(4, GL_UNSIGNED_BYTE, 0, *);
+        // glInterleavedArrays(GL_T2F_N3F_V3F, 0, *)
+        glPushAttrib (GL_ALL_ATTRIB_BITS);
+
+
+#ifdef FIX_I810_LOD_BIAS
+	// If 2D textures weren't previously enabled, enable
+	// them now and force the driver to notice the update,
+	// then disable them again.
+	if (!glIsEnabled(GL_TEXTURE_2D)) {
+	  // Clearing a mask of zero *should* have no
+	  // side effects, but coupled with enbling
+	  // GL_TEXTURE_2D it works around a segmentation
+	  // fault in the driver for the Intel 810 chip.
+	  glEnable(GL_TEXTURE_2D);
+	  glClear(0);
+	  glDisable(GL_TEXTURE_2D);
+	}
+#endif
+
     return new render_handler_ogl;
 }
 
