@@ -24,19 +24,24 @@
 // code, see the FIST web page at:
 // http://www.cosy.sbg.ac.at/~held/projects/triang/triang.html
 
+/* $Id: triangulate_impl.h,v 1.23 2007/04/27 07:51:33 strk Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "utility.h"
 #include "triangulate.h"
 #include "tu_random.h"
 #include "grid_index.h"
+#include "log.h"
 
 #define PROFILE_TRIANGULATE
 #ifdef PROFILE_TRIANGULATE
 #include "tu_timer.h"
 #endif // PROFILE_TRIANGULATE
 
-
-// Template this whole thing on coord_t, so sint16 and float versions
+// Template this whole thing on coord_t, so int16_t and float versions
 // can easily reuse the code.
 //
 // These templates are instantiated in triangulate_<type>.cpp files.
@@ -44,12 +49,13 @@
 // unused types.
 
 
-// convenience struct; this could use some public vec2 type, but often
+// convenience class; this could use some public vec2 type, but often
 // it's nicer for users if the external interface is smaller and more
 // c-like, since they probably have their own vec2 that they prefer.
 template<class coord_t>
-struct vec2
+class vec2
 {
+public:
 	vec2() : x(0), y(0) {}
 	vec2(coord_t _x, coord_t _y) : x(_x), y(_y) {}
 
@@ -70,10 +76,10 @@ inline double	determinant_float(const vec2<float>& a, const vec2<float>& b, cons
 }
 
 
-inline sint64	determinant_sint32(const vec2<sint32>& a, const vec2<sint32>& b, const vec2<sint32>& c)
+inline int64_t	determinant_sint32(const vec2<int32_t>& a, const vec2<int32_t>& b, const vec2<int32_t>& c)
 {
-	return (sint64(b.x) - sint64(a.x)) * (sint64(c.y) - sint64(a.y))
-		- (sint64(b.y) - sint64(a.y)) * (sint64(c.x) - sint64(a.x));
+	return (int64_t(b.x) - int64_t(a.x)) * (int64_t(c.y) - int64_t(a.y))
+		- (int64_t(b.y) - int64_t(a.y)) * (int64_t(c.x) - int64_t(a.x));
 }
 
 
@@ -99,10 +105,10 @@ inline int	vertex_left_test(const vec2<float>& a, const vec2<float>& b, const ve
 
 
 template<>
-inline int	vertex_left_test(const vec2<sint32>& a, const vec2<sint32>& b, const vec2<sint32>& c)
-// Specialize for vec2<sint32>
+inline int	vertex_left_test(const vec2<int32_t>& a, const vec2<int32_t>& b, const vec2<int32_t>& c)
+// Specialize for vec2<int32_t>
 {
-	sint64	det = determinant_sint32(a, b, c);
+	int64_t	det = determinant_sint32(a, b, c);
 	if (det > 0) return 1;
 	else if (det < 0) return -1;
 	else return 0;
@@ -131,7 +137,7 @@ bool	vertex_in_ear(const vec2<coord_t>& v, const vec2<coord_t>& a, const vec2<co
 }
 
 
-template<class coord_t> struct poly;
+template<class coord_t> class poly;
 
 
 
@@ -159,8 +165,9 @@ inline int	remap_index_for_duped_verts(int index, int duped_v0, int duped_v1)
 
 
 template<class coord_t>
-struct poly_vert
+class poly_vert
 {
+public:
 	poly_vert() {}
 	poly_vert(coord_t x, coord_t y, poly<coord_t>* owner, int my_index)
 		:
@@ -295,10 +302,10 @@ inline bool	edges_intersect_sub(const std::vector<poly_vert<float> >& sorted_ver
 
 
 template<>
-inline bool	edges_intersect_sub(const std::vector<poly_vert<sint32> >& sorted_verts, int e0v0i, int e0v1i, int e1v0i, int e1v1i)
+inline bool	edges_intersect_sub(const std::vector<poly_vert<int32_t> >& sorted_verts, int e0v0i, int e0v1i, int e1v0i, int e1v1i)
 // Return true if edge (e0v0,e0v1) intersects (e1v0,e1v1).
 //
-// Specialized for sint32
+// Specialized for int32_t
 {
 	// If e1v0,e1v1 are on opposite sides of e0, and e0v0,e0v1 are
 	// on opposite sides of e1, then the segments cross.  These
@@ -309,10 +316,10 @@ inline bool	edges_intersect_sub(const std::vector<poly_vert<sint32> >& sorted_ve
 	//
 	// If only one is degenerate, our tests are still OK.
 
-	const vec2<sint32>&	e0v0 = sorted_verts[e0v0i].m_v;
-	const vec2<sint32>&	e0v1 = sorted_verts[e0v1i].m_v;
-	const vec2<sint32>&	e1v0 = sorted_verts[e1v0i].m_v;
-	const vec2<sint32>&	e1v1 = sorted_verts[e1v1i].m_v;
+	const vec2<int32_t>&	e0v0 = sorted_verts[e0v0i].m_v;
+	const vec2<int32_t>&	e0v1 = sorted_verts[e0v1i].m_v;
+	const vec2<int32_t>&	e1v0 = sorted_verts[e1v0i].m_v;
+	const vec2<int32_t>&	e1v1 = sorted_verts[e1v1i].m_v;
 
 	if (e0v0.x == e0v1.x && e0v0.y == e0v1.y)
 	{
@@ -327,8 +334,8 @@ inline bool	edges_intersect_sub(const std::vector<poly_vert<sint32> >& sorted_ve
 	}
 
 	// See if e1 crosses line of e0.
-	sint64	det10 = determinant_sint32(e0v0, e0v1, e1v0);
-	sint64	det11 = determinant_sint32(e0v0, e0v1, e1v1);
+	int64_t	det10 = determinant_sint32(e0v0, e0v1, e1v0);
+	int64_t	det11 = determinant_sint32(e0v0, e0v1, e1v1);
 
 	// Note: we do > 0, which means a vertex on a line counts as
 	// intersecting.  In general, if one vert is on the other
@@ -343,8 +350,8 @@ inline bool	edges_intersect_sub(const std::vector<poly_vert<sint32> >& sorted_ve
 	}
 
 	// See if e0 crosses line of e1.
-	sint64	det00 = determinant_sint32(e1v0, e1v1, e0v0);
-	sint64	det01 = determinant_sint32(e1v0, e1v1, e0v1);
+	int64_t	det00 = determinant_sint32(e1v0, e1v1, e0v0);
+	int64_t	det01 = determinant_sint32(e1v0, e1v1, e0v1);
 
 	if (det00 * det01 > 0)
 	{
@@ -408,8 +415,9 @@ bool	is_convex_vert(const std::vector<poly_vert<coord_t> >& sorted_verts, int vi
 
 
 template<class coord_t>
-struct poly
+class poly
 {
+public:
 	typedef poly_vert<coord_t> vert_t;
 
 	poly(/*@@ TODO std::vector<vert_t>* sorted_verts*/)
@@ -841,7 +849,7 @@ void	poly<coord_t>::dirty_vert(std::vector<vert_t>* sorted_verts, int vi)
 
 
 template<class coord_t>
-bool	poly<coord_t>::build_ear_list(std::vector<vert_t>* sorted_verts, tu_random::generator* rg)
+bool	poly<coord_t>::build_ear_list(std::vector<vert_t>* sorted_verts, tu_random::generator* /* rg */)
 // Initialize our ear loop with all the ears that can be clipped.
 //
 // Returns true if we clipped any degenerates while looking for ears.
@@ -930,7 +938,7 @@ bool	poly<coord_t>::build_ear_list(std::vector<vert_t>* sorted_verts, tu_random:
 
 
 template<class coord_t>
-int	poly<coord_t>::get_next_ear(const std::vector<vert_t>& sorted_verts, tu_random::generator* rg)
+int	poly<coord_t>::get_next_ear(const std::vector<vert_t>& sorted_verts, tu_random::generator* /* rg */)
 // Return the next ear to be clipped.
 {
 	assert(m_ear_count > 0);
@@ -1783,16 +1791,16 @@ bool	poly<coord_t>::vert_is_duplicated(const std::vector<vert_t>& sorted_verts,
 
 
 template<class coord_t>
-struct poly_env
+class poly_env
 // Struct that holds the state of a triangulation.
 {
+public:
 //data:
 	std::vector<poly_vert<coord_t> >	m_sorted_verts;
 	std::vector<poly<coord_t>*>	m_polys;
 
 	index_box<coord_t>	m_bound;
 	int	m_estimated_triangle_count;
-
 //code:
 	void	init(int path_count, const std::vector<coord_t> paths[]);
 	void	join_paths_into_one_poly();
@@ -1869,8 +1877,9 @@ void	poly_env<coord_t>::init(int path_count, const std::vector<coord_t> paths[])
 		if (path.size() & 1)
 		{
 			// Bad input, odd number of coords.
-			assert(0);
-			fprintf(stderr, "path[%d] has odd number of coords (%zd), dropping last value\n", i, path.size());//xxxx
+			assert(0);			
+			fprintf(stderr, "path[%d] has odd number of coords (" SIZET_FMT \
+			        "), dropping last value\n", i, path.size());//xxxx
 			path_size--;
 		}
 		for (int j = 0; j < path_size; j += 2)	// vertex coords come in pairs.
@@ -2217,7 +2226,7 @@ static void compute_triangulation(
 	}
 
 #ifdef PROFILE_TRIANGULATE
-	uint64	start_ticks = tu_timer::get_profile_ticks();
+	uint64_t	start_ticks = tu_timer::get_profile_ticks();
 #endif // PROFILE_TRIANGULATE
 
 	// Local generator, for some parts of the algo that need random numbers.
@@ -2239,7 +2248,7 @@ static void compute_triangulation(
 	}
 
 #ifdef PROFILE_TRIANGULATE
-	uint64	join_ticks = tu_timer::get_profile_ticks();
+	uint64_t	join_ticks = tu_timer::get_profile_ticks();
 	fprintf(stderr, "join poly = %1.6f sec\n", tu_timer::profile_ticks_to_seconds(join_ticks - start_ticks));
 #endif // PROFILE_TRIANGULATE
 
@@ -2372,7 +2381,7 @@ static void compute_triangulation(
 	}
 	
 #ifdef PROFILE_TRIANGULATE
-	uint64	clip_ticks = tu_timer::get_profile_ticks();
+	uint64_t	clip_ticks = tu_timer::get_profile_ticks();
 	fprintf(stderr, "clip poly = %1.6f sec\n", tu_timer::profile_ticks_to_seconds(clip_ticks - join_ticks));
 	fprintf(stderr, "total for poly = %1.6f sec\n", tu_timer::profile_ticks_to_seconds(clip_ticks - start_ticks));
 	fprintf(stderr, "vert count = %d, verts clipped / sec = %f\n",
@@ -2389,7 +2398,7 @@ static void compute_triangulation(
 
 template<class coord_t>
 void	recovery_process(
-	std::vector<poly<coord_t>*>* polys,
+                         std::vector<poly<coord_t>*>* /* polys */,
 	poly<coord_t>* P,
 	std::vector<poly_vert<coord_t> >* sorted_verts,
 	tu_random::generator* rg)
