@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,30 +10,11 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// Linking Gnash statically or dynamically with other modules is making a
-// combined work based on Gnash. Thus, the terms and conditions of the GNU
-// General Public License cover the whole combination.
-//
-// As a special exception, the copyright holders of Gnash give you
-// permission to combine Gnash with free software programs or libraries
-// that are released under the GNU LGPL and with code included in any
-// release of Talkback distributed by the Mozilla Foundation. You may
-// copy and distribute such a system following the terms of the GNU GPL
-// for all but the LGPL-covered parts and Talkback, and following the
-// LGPL for the LGPL-covered parts.
-//
-// Note that people who make modified versions of Gnash are not obligated
-// to grant this special exception for their modified versions; it is their
-// choice whether to do so. The GNU General Public License gives permission
-// to release a modified version without this exception; this exception
-// also makes it possible to release a modified version which carries
-// forward this exception.
-// 
-//
 //
 // Written by Koos Vriezen <koos ! vriezen ? xs4all ! nl>
 
@@ -42,6 +23,7 @@
 #endif
 
 #include <config.h>
+#include <cassert>
 #include <qcstring.h>
 #include <qpopupmenu.h>
 #include <qtimer.h>
@@ -64,9 +46,9 @@ class KXMLGUIClient; // workaround for kde3.3 on sarge with gcc4, kactioncollect
 
 class KlashFactory : public KParts::Factory {
 public:
-    KlashFactory ();
-    virtual ~KlashFactory ();
-    virtual KParts::Part *createPartObject
+    KDE_NO_CDTOR_EXPORT KlashFactory ();
+    KDE_NO_CDTOR_EXPORT virtual ~KlashFactory ();
+    KDE_NO_CDTOR_EXPORT virtual KParts::Part *createPartObject
         (QWidget *wparent, const char *wname,
          QObject *parent, const char *name,
          const char *className, const QStringList &args);
@@ -79,15 +61,15 @@ K_EXPORT_COMPONENT_FACTORY (libklashpart, KlashFactory)
 
 KInstance *KlashFactory::s_instance = 0;
 
-KDE_NO_CDTOR_EXPORT KlashFactory::KlashFactory () {
+KlashFactory::KlashFactory () {
     s_instance = new KInstance ("klash");
 }
 
-KDE_NO_CDTOR_EXPORT KlashFactory::~KlashFactory () {
+KlashFactory::~KlashFactory () {
     delete s_instance;
 }
 
-KDE_NO_EXPORT KParts::Part *KlashFactory::createPartObject
+KParts::Part *KlashFactory::createPartObject
   (QWidget *wparent, const char *wname,
    QObject *parent, const char * name,
    const char * cls, const QStringList & args) {
@@ -172,7 +154,18 @@ KDE_NO_EXPORT bool KlashPart::allowRedir (const KURL & url) const {
     return kapp->authorizeURLAction ("redirect", m_docbase, url);
 }
 
-KDE_NO_EXPORT void KlashPart::play () {
+KDE_NO_EXPORT void KlashPart::play ()
+{
+
+    QString procname;
+    char *gnash_env = getenv("KLASH_PLAYER");
+    if (!gnash_env) {
+      procname = GNASHBINDIR;
+      procname += "/klash";
+    } else {
+      procname = gnash_env;
+    }
+
     stop ();
     if (m_src_url.isEmpty ())
         return;
@@ -183,7 +176,8 @@ KDE_NO_EXPORT void KlashPart::play () {
     if (m_width > 0 && m_height > 0)
         dim = QString ("-j ") + QString::number (m_width) +
             QString (" -k ") + QString::number (m_height);
-    QString cmd = QString ("klash -x ") +
+    // TODO: check KLASH_PLAYER env ?
+    QString cmd = procname + QString (" -x ") +
         QString::number (static_cast <KlashView *> (widget ())->embedId()) +
         QChar (' ') + dim +
         QChar (' ') + KProcess::quote (m_src_url);
@@ -202,8 +196,17 @@ KDE_NO_EXPORT void KlashPart::stop () {
         if (m_process->isRunning ()) {
             ; // IPC close
             //m_process->wait(2);
+	    
+	    // Ignore SIGTERM, so we won't kill ourselves.
             void (*oldhandler)(int) = signal(SIGTERM, SIG_IGN);
-            ::kill (-1 * ::getpid (), SIGTERM);
+
+	    int pid = -1 * ::getpid();
+	    assert(pid < -1);
+
+	    // Terminate every process in our process group.
+            ::kill (pid, SIGTERM);
+
+	    // Restore the old handler.
             signal(SIGTERM, oldhandler);
             m_process->wait(2);
         }
@@ -544,12 +547,7 @@ static const int XKeyPress = KeyPress;
 #undef Unsorted
 #undef Bool
 
-KlashEmbed::KlashEmbed (KlashView * view) : QXEmbed (view), m_view (view) {
-    setProtocol (QXEmbed::XPLAIN);
-    int scr = DefaultScreen (qt_xdisplay ());
-    embed (XCreateSimpleWindow (qt_xdisplay(), view->winId (), 0, 0, width(), height(), 1, BlackPixel (qt_xdisplay(), scr), BlackPixel (qt_xdisplay(), scr)));
-    XClearWindow (qt_xdisplay(), embeddedWinId ());
-}
+KlashEmbed::KlashEmbed (KlashView * view) : QXEmbed (view), m_view (view) {}
 
 KlashEmbed::~KlashEmbed () {}
 
@@ -565,7 +563,7 @@ void KlashView::resizeEvent (QResizeEvent *) {
 }
 
 WId KlashView::embedId () {
-    return m_embed->embeddedWinId ();
+    return m_embed->winId ();
 }
 
 #include "klash_part.moc"
