@@ -1,10 +1,11 @@
-// fontlib.cpp	-- Thatcher Ulrich <tu@tulrich.com> 2003
+
 
 // This source code has been donated to the Public Domain.  Do
 // whatever you want with it.
 
 // A module to take care of all of gnash's loaded fonts.
 
+/* $Id: fontlib.cpp,v 1.26 2006/11/11 22:44:54 strk Exp $ */
 
 #include "container.h"
 #include "tu_file.h"
@@ -13,16 +14,17 @@
 #include "impl.h"
 #include "log.h"
 #include "render.h"
-#include "shape.h"
+//#include "shape.h"
+#include "shape_character_def.h"
 #include "styles.h"
 #include "tesselate.h"
 #include "render.h"
-#include "Movie.h"
+#include "movie_definition.h"
 
 
 namespace gnash {
 namespace fontlib {
-	std::vector< smart_ptr<font> >	s_fonts;
+	std::vector< boost::intrusive_ptr<font> >	s_fonts;
 
 	// Size (in TWIPS) of the box that the glyph should
 	// stay within.
@@ -92,14 +94,15 @@ namespace fontlib {
 	// State for the glyph packer.
 	//
 
-	static Uint8*	s_render_buffer = NULL;
+	static uint8_t*	s_render_buffer = NULL;
 	static matrix	s_render_matrix;
 
-	static Uint8*	s_current_cache_image = NULL;
+	static uint8_t*	s_current_cache_image = NULL;
 
 	// for setting the bitmap_info after they're packed.
-	struct pending_glyph_info
+	class pending_glyph_info
 	{
+	public:
 		font*	m_source_font;
 		int	m_glyph_index;
 		texture_glyph	m_texture_glyph;
@@ -123,8 +126,12 @@ namespace fontlib {
 
 
 	// Integer-bounded 2D rectangle.
-	struct recti
+	// 
+	// TODO: make the gnash::rect a templated class instead
+	//
+	class recti
 	{
+	public:
 		int	m_x_min, m_x_max, m_y_min, m_y_max;
 
 		recti(int x0 = 0, int x1 = 0, int y0 = 0, int y1 = 0)
@@ -169,8 +176,9 @@ namespace fontlib {
 	static std::vector<recti>	s_covered_rects;
 
 	// 2d integer point.
-	struct pointi
+	class pointi
 	{
+	public:
 		int	m_x, m_y;
 
 		pointi(int x = 0, int y = 0)
@@ -203,7 +211,7 @@ namespace fontlib {
 			// Set up a cache.
 			if (s_current_cache_image == NULL)
 			{
-				s_current_cache_image = new Uint8[GLYPH_CACHE_TEXTURE_SIZE * GLYPH_CACHE_TEXTURE_SIZE];
+				s_current_cache_image = new uint8_t[GLYPH_CACHE_TEXTURE_SIZE * GLYPH_CACHE_TEXTURE_SIZE];
 			}
 			memset(s_current_cache_image, 0, GLYPH_CACHE_TEXTURE_SIZE * GLYPH_CACHE_TEXTURE_SIZE);
 
@@ -266,32 +274,29 @@ namespace fontlib {
 			}
 		}
 
-		smart_ptr<bitmap_info>	bi;
-		if (owner->get_create_bitmaps() == DO_NOT_LOAD_BITMAPS)
-		{
-			bi = render::create_bitmap_info_empty();
-		}
-		else
+		if (owner->get_create_bitmaps() == DO_LOAD_BITMAPS)
 		{
+			boost::intrusive_ptr<bitmap_info>	bi;
 			bi = render::create_bitmap_info_alpha(
 				GLYPH_CACHE_TEXTURE_SIZE,
 				GLYPH_CACHE_TEXTURE_SIZE,
 				s_current_cache_image);
-		}
-		owner->add_bitmap_info(bi.get_ptr());
+			owner->add_bitmap_info(bi.get());
 
-		// Push finished glyphs into their respective fonts.
-		for (int i = 0, n = s_pending_glyphs.size(); i < n; i++)
-		{
+			// Push finished glyphs into their respective fonts.
+			for (int i = 0, n = s_pending_glyphs.size(); i < n; i++)
+			{
 			pending_glyph_info*	pgi = &s_pending_glyphs[i];
 			assert(pgi->m_glyph_index != -1);
 			assert(pgi->m_source_font != NULL);
 
-			pgi->m_texture_glyph.set_bitmap_info(bi.get_ptr());
+			pgi->m_texture_glyph.set_bitmap_info(bi.get());
 			pgi->m_source_font->add_texture_glyph(pgi->m_glyph_index, pgi->m_texture_glyph);
-			//s_pending_glyphs[i]->set_bitmap_info(bi.get_ptr());
+			//s_pending_glyphs[i]->set_bitmap_info(bi.get());
+			}
 		}
 		s_pending_glyphs.clear();
+		
 	}
 
 
@@ -431,8 +436,9 @@ namespace fontlib {
 
 	// This is for keeping track of our rendered glyphs, before
 	// packing them into textures and registering with the font.
-	struct rendered_glyph_info
+	class rendered_glyph_info
 	{
+	public:
 		font*	m_source_font;
 		int	m_glyph_index;
 		image::alpha*	m_image;
@@ -461,8 +467,8 @@ namespace fontlib {
 	{
 		assert(s_render_buffer);
 
-		int	iy0 = (int) ceilf(y0);
-		int	iy1 = (int) ceilf(y1);
+		int	iy0 = static_cast<int>(ceilf(y0));
+		int	iy1 = static_cast<int>(ceilf(y1));
 		float	dy = y1 - y0;
 
 		for (int y = iy0; y < iy1; y++)
@@ -471,8 +477,8 @@ namespace fontlib {
 			if (y >= s_glyph_render_size) return;
 
 			float	f = (y - y0) / dy;
-			int	xl = (int) ceilf(flerp(xl0, xl1, f));
-			int	xr = (int) ceilf(flerp(xr0, xr1, f));
+			int	xl = static_cast<int>(ceilf(flerp(xl0, xl1, f)));
+			int	xr = static_cast<int>(ceilf(flerp(xr0, xr1, f)));
 			
 			xl = iclamp(xl, 0, s_glyph_render_size - 1);
 			xr = iclamp(xr, 0, s_glyph_render_size - 1);
@@ -487,12 +493,13 @@ namespace fontlib {
 	}
 
 
-	struct draw_into_software_buffer : tesselate::trapezoid_accepter
 	// A trapezoid accepter that does B&W rendering into our
 	// software buffer.
+        class draw_into_software_buffer : public tesselate::trapezoid_accepter
 	{
+	public:
 		// Overrides from trapezoid_accepter
-		virtual void	accept_trapezoid(int style, const tesselate::trapezoid& tr)
+		virtual void	accept_trapezoid(int /* style */, const tesselate::trapezoid& tr)
 		{
 			// Transform the coords.
 			float	x_scale = s_render_matrix.m_[0][0];
@@ -511,14 +518,16 @@ namespace fontlib {
 			software_trapezoid(y0, y1, lx0, lx1, rx0, rx1);
 		}
 
-		virtual void	accept_line_strip(int style, const point coords[], int coord_count)
+		virtual void	accept_line_strip(int /*style*/,
+				const point* /*coords*/, int /*coord_count*/)
 		{
-			assert(0);	// Shape glyphs should not contain lines.
+			// Shape glyphs should not contain lines.
+			assert(0);
 		}
 	};
 
 
-	static bool	render_glyph(rendered_glyph_info* rgi, const shape_character_def* sh)
+	static bool	render_glyph(rendered_glyph_info& rgi, const shape_character_def* sh)
 	// Render the given outline shape into a cached font texture.
 	// Return true if the glyph is not empty; false if it's
 	// totally empty.
@@ -526,7 +535,6 @@ namespace fontlib {
 	// Return fill in the image and offset members of the given
 	// rgi.
 	{
-		assert(rgi);
 		assert(sh);
 		assert(s_render_buffer);
 
@@ -543,13 +551,13 @@ namespace fontlib {
 		float	offset_y = s_rendering_box;
 		rect	glyph_bounds;
 		sh->compute_bound(&glyph_bounds);
-		if (glyph_bounds.m_x_min < 0)
+		if (glyph_bounds.get_x_min() < 0)
 		{
-			offset_x = - glyph_bounds.m_x_min;
+			offset_x = - glyph_bounds.get_x_min();
 		}
-		if (glyph_bounds.m_y_max > 0)
+		if (glyph_bounds.get_y_max() > 0)
 		{
-			offset_y = s_rendering_box - glyph_bounds.m_y_max;
+			offset_y = s_rendering_box - glyph_bounds.get_y_max();
 		}
 
 		s_render_matrix.set_identity();
@@ -571,7 +579,7 @@ namespace fontlib {
 		int	max_x = 0;
 		int	min_y = s_glyph_nominal_size;
 		int	max_y = 0;
-		Uint8*	output = new Uint8[s_glyph_nominal_size * s_glyph_nominal_size];
+		uint8_t*	output = new uint8_t[s_glyph_nominal_size * s_glyph_nominal_size];
 		for (int j = 0; j < s_glyph_nominal_size; j++)
 		{
 			for (int i = 0; i < s_glyph_nominal_size; i++)
@@ -582,7 +590,7 @@ namespace fontlib {
 				{
 					for (int ii = 0; ii < OVERSAMPLE_FACTOR; ii++)
 					{
-						Uint8	texel = s_render_buffer[
+						uint8_t	texel = s_render_buffer[
 							((j << OVERSAMPLE_BITS) + jj) * s_glyph_render_size
 							+ ((i << OVERSAMPLE_BITS) + ii)];
 						sum += texel;
@@ -598,24 +606,24 @@ namespace fontlib {
 					min_y = imin(min_y, j);
 					max_y = imax(max_y, j);
 				}
-				output[j * s_glyph_nominal_size + i] = (Uint8) sum;
+				output[j * s_glyph_nominal_size + i] = (uint8_t) sum;
 			}
 		}
 
 		if (any_nonzero_pixels)
 		{
 			// Fill in rendered_glyph_info.
-			rgi->m_image = new image::alpha(max_x - min_x + 1, max_y - min_y + 1);
-			rgi->m_offset_x = offset_x / s_rendering_box * s_glyph_nominal_size - min_x;
-			rgi->m_offset_y = offset_y / s_rendering_box * s_glyph_nominal_size - min_y;
+			rgi.m_image = new image::alpha(max_x - min_x + 1, max_y - min_y + 1);
+			rgi.m_offset_x = offset_x / s_rendering_box * s_glyph_nominal_size - min_x;
+			rgi.m_offset_y = offset_y / s_rendering_box * s_glyph_nominal_size - min_y;
 
 			// Copy the rendered glyph into the new image.
-			{for (int j = 0, n = rgi->m_image->m_height; j < n; j++)
+			{for (int j = 0, n = rgi.m_image->m_height; j < n; j++)
 			{
 				memcpy(
-					image::scanline(rgi->m_image, j),
+					image::scanline(rgi.m_image, j),
 					output + (min_y + j) * s_glyph_nominal_size + min_x,
-					rgi->m_image->m_width);
+					rgi.m_image->m_width);
 			}}
 		}
 		else
@@ -626,7 +634,7 @@ namespace fontlib {
 
 		delete [] output;	// @@ TODO should keep this around longer, instead of new/delete for each glyph
 
-		rgi->m_image_hash = rgi->m_image->compute_hash();
+		rgi.m_image_hash = rgi.m_image->compute_hash();
 
 		return true;
 	}
@@ -689,9 +697,9 @@ namespace fontlib {
 				tg = identical_tg;
 
 				// Use our own offset, in case it's different.
-				tg.m_uv_origin.m_x = tg.m_uv_bounds.m_x_min
+				tg.m_uv_origin.m_x = tg.m_uv_bounds.get_x_min()
 					+ rgi.m_offset_x / GLYPH_CACHE_TEXTURE_SIZE;
-				tg.m_uv_origin.m_y = tg.m_uv_bounds.m_y_min
+				tg.m_uv_origin.m_y = tg.m_uv_bounds.get_y_min()
 					+ rgi.m_offset_y / GLYPH_CACHE_TEXTURE_SIZE;
 
 				if (identical_tg.is_renderable())
@@ -742,7 +750,7 @@ namespace fontlib {
 	}
 
 
-	void	pack_and_assign_glyphs(std::vector<rendered_glyph_info>* glyph_info, movie_definition* owner)
+	void	pack_and_assign_glyphs(std::vector<rendered_glyph_info>& glyph_info, movie_definition* owner)
 	// Pack the given glyphs into textures, and push the
 	// texture_glyph info into the source fonts.
 	//
@@ -750,8 +758,9 @@ namespace fontlib {
 	// otherwise doesn't munge the array.
 	{
 		// Sort the glyphs by size (biggest first).
-		struct sorter
+		class sorter
 		{
+		public:
 			static int	sort_by_size(const void* a, const void* b)
 			// For qsort.
 			{
@@ -764,32 +773,27 @@ namespace fontlib {
 				return b_size - a_size;
 			}
 		};
-		if (glyph_info->size())
+		if (glyph_info.size())
 		{
-			qsort(&(*glyph_info)[0], glyph_info->size(), sizeof((*glyph_info)[0]), sorter::sort_by_size);
+			qsort(&glyph_info[0], glyph_info.size(), sizeof(glyph_info[0]), sorter::sort_by_size);
 		}
 
 		// Flag for whether we've processed this glyph yet.
-		std::vector<bool>	packed;
-		packed.resize(glyph_info->size());
-		for (int i = 0, n = packed.size(); i < n; i++)
-		{
-			packed[i] = false;
-		}
+		std::vector<bool> packed(glyph_info.size(), false);
 
 		// Share identical texture data where possible, by
 		// doing glyph image comparisons.
 		hash<unsigned int, const rendered_glyph_info*>	image_hash;
 
 		// Pack the glyphs.
-		{for (int i = 0, n = glyph_info->size(); i < n; )
+		{for (int i = 0, n = glyph_info.size(); i < n; )
 		{
 			int	index = i;
 
 			// Try to pack a glyph into the existing texture.
 			for (;;)
 			{
-				const rendered_glyph_info&	rgi = (*glyph_info)[index];
+				const rendered_glyph_info&	rgi = glyph_info[index];
 
 				// First things first: are we identical to a glyph that has
 				// already been packed?
@@ -829,10 +833,14 @@ namespace fontlib {
 					texture_glyph	tg;
 					tg.m_uv_origin.m_x = (pack_x + rgi.m_offset_x) / (GLYPH_CACHE_TEXTURE_SIZE);
 					tg.m_uv_origin.m_y = (pack_y + rgi.m_offset_y) / (GLYPH_CACHE_TEXTURE_SIZE);
-					tg.m_uv_bounds.m_x_min = float(pack_x) / (GLYPH_CACHE_TEXTURE_SIZE);
-					tg.m_uv_bounds.m_x_max = float(pack_x + width) / (GLYPH_CACHE_TEXTURE_SIZE);
-					tg.m_uv_bounds.m_y_min = float(pack_y) / (GLYPH_CACHE_TEXTURE_SIZE);
-					tg.m_uv_bounds.m_y_max = float(pack_y + height) / (GLYPH_CACHE_TEXTURE_SIZE);
+					tg.m_uv_bounds.enclose_point(
+						float(pack_x) / (GLYPH_CACHE_TEXTURE_SIZE),
+						float(pack_y) / (GLYPH_CACHE_TEXTURE_SIZE)
+					);
+					tg.m_uv_bounds.expand_to_point(
+						float(pack_x + width) / (GLYPH_CACHE_TEXTURE_SIZE),
+						float(pack_y + height) / (GLYPH_CACHE_TEXTURE_SIZE)
+					);
 
 					// Fill in bitmap info and push into the source font later.
 					s_pending_glyphs.push_back(
@@ -874,11 +882,10 @@ namespace fontlib {
 	}
 
 
-	static void	generate_font_bitmaps(std::vector<rendered_glyph_info>* glyph_info, font* f, movie_definition* owner)
+static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info, font* f, movie_definition* /* owner */)
 	// Render images for each of the font's glyphs, and put the
 	// info about them in the given array.
 	{
-		assert(glyph_info);
 		assert(f);
 
 		f->set_texture_glyph_nominal_size(s_glyph_nominal_size);
@@ -905,9 +912,9 @@ namespace fontlib {
 						rgi.m_source_font = f;
 						rgi.m_glyph_index = i;
 
-						if (render_glyph(&rgi, sh) == true)
+						if (render_glyph(rgi, sh) == true)
 						{
-							glyph_info->push_back(rgi);
+							glyph_info.push_back(rgi);
 						}
 						// else glyph is empty
 					}
@@ -942,17 +949,17 @@ namespace fontlib {
 	// Build cached textures from glyph outlines.
 	{
 		assert(s_render_buffer == NULL);
-		s_render_buffer = new Uint8[s_glyph_render_size * s_glyph_render_size];
+		s_render_buffer = new uint8_t[s_glyph_render_size * s_glyph_render_size];
 
 		// Build the glyph images.
 		std::vector<rendered_glyph_info>	glyph_info;
 		for (unsigned int i = 0; i < fonts.size(); i++)
 		{
-			generate_font_bitmaps(&glyph_info, fonts[i], owner);
+			generate_font_bitmaps(glyph_info, fonts[i], owner);
 		}
 
 		// Pack all the rendered glyphs and push the info into their fonts.
-		pack_and_assign_glyphs(&glyph_info, owner);
+		pack_and_assign_glyphs(glyph_info, owner);
 
 		// Delete glyph images.
 		{for (int i = 0, n = glyph_info.size(); i < n; i++)
@@ -1055,13 +1062,13 @@ namespace fontlib {
 					assert(bi >= bitmaps_used_base
 					       && bi < owner->get_bitmap_info_count());
 
-					out->write_le16((Uint16) (bi - bitmaps_used_base));
+					out->write_le16((uint16_t) (bi - bitmaps_used_base));
 
 					// save rect, position.
-					out->write_float32(tg.m_uv_bounds.m_x_min);
-					out->write_float32(tg.m_uv_bounds.m_y_min);
-					out->write_float32(tg.m_uv_bounds.m_x_max);
-					out->write_float32(tg.m_uv_bounds.m_y_max);
+					out->write_float32(tg.m_uv_bounds.get_x_min());
+					out->write_float32(tg.m_uv_bounds.get_y_min());
+					out->write_float32(tg.m_uv_bounds.get_x_max());
+					out->write_float32(tg.m_uv_bounds.get_y_max());
 					out->write_float32(tg.m_uv_origin.m_x);
 					out->write_float32(tg.m_uv_origin.m_y);
 					n++;
@@ -1103,22 +1110,14 @@ namespace fontlib {
 			int w = in->read_le16();
 			int h = in->read_le16();
 
-			smart_ptr<bitmap_info>	bi;
-			if (owner->get_create_bitmaps() == DO_NOT_LOAD_BITMAPS)
-			{
-				// Skip image data bytes.
-				in->set_position(in->get_position() + w * h);
-
-				// Make a placeholder bitmap.
-				bi = render::create_bitmap_info_empty();
-			}
-			else
+			if (owner->get_create_bitmaps() == DO_LOAD_BITMAPS)
 			{
+			boost::intrusive_ptr<bitmap_info>	bi;
 				// load bitmap contents
 				if (s_current_cache_image == NULL || w != pw || h != ph)
 				{
 					delete [] s_current_cache_image;
-					s_current_cache_image = new Uint8[w*h];
+					s_current_cache_image = new uint8_t[w*h];
 					pw = w;
 					ph = h;
 				}
@@ -1129,9 +1128,12 @@ namespace fontlib {
 					w,
 					h,
 					s_current_cache_image);
-			}
-			owner->add_bitmap_info(bi.get_ptr());
+			owner->add_bitmap_info(bi.get());
 			assert(bi->get_ref_count() == 2);	// one ref for bi, one for the owner.
+			}
+			else { 	// Skip image data bytes.
+				in->set_position(in->get_position() + w * h);
+				}
 		}
 
 		delete [] s_current_cache_image;
@@ -1142,7 +1144,7 @@ namespace fontlib {
 		if (nf != (int) fonts.size())
 		{
 			// Font counts must match!
-			log_error("error: mismatched font count (read %d, expected %zd) in cached font data\n", nf, fonts.size());
+			log_error("mismatched font count (read %d, expected " SIZET_FMT ") in cached font data\n", nf, fonts.size());
 			in->go_to_end();
 			goto error_exit;
 		}
@@ -1183,7 +1185,7 @@ namespace fontlib {
 				if (bi + bitmaps_used_base >= owner->get_bitmap_info_count())
 				{
 					// Bad data; give up.
-					log_error("error: invalid bitmap index %d in cached font data\n", bi);
+					log_error("invalid bitmap index %d in cached font data\n", bi);
 					in->go_to_end();
 					goto error_exit;
 				}
@@ -1191,17 +1193,19 @@ namespace fontlib {
 				tg.set_bitmap_info(owner->get_bitmap_info(bi + bitmaps_used_base));
 
 				// load glyph bounds and origin.
-				tg.m_uv_bounds.m_x_min = in->read_float32();
-				tg.m_uv_bounds.m_y_min = in->read_float32();
-				tg.m_uv_bounds.m_x_max = in->read_float32();
-				tg.m_uv_bounds.m_y_max = in->read_float32();
+				float xmin = in->read_float32();
+				float ymin = in->read_float32();
+				float xmax = in->read_float32();
+				float ymax = in->read_float32();
+				tg.m_uv_bounds.enclose_point(xmin, ymin);
+				tg.m_uv_bounds.expand_to_point(xmax, ymax);
 				tg.m_uv_origin.m_x = in->read_float32();
 				tg.m_uv_origin.m_y = in->read_float32();
 
 				if (glyph_index < 0 || glyph_index >= fnt->get_glyph_count())
 				{
 					// Cached data doesn't match this font!
-					log_error("error: invalid glyph index %d in cached font data, limit is %d, font is '%s'\n",
+					log_error("invalid glyph index %d in cached font data, limit is %d, font is '%s'\n",
 						  glyph_index,
 						  fnt->get_glyph_count(),
 						  fnt->get_name());
@@ -1244,7 +1248,7 @@ namespace fontlib {
 			return NULL;
 		}
 
-		return s_fonts[index].get_ptr();
+		return s_fonts[index].get();
 	}
 
 
@@ -1254,7 +1258,7 @@ namespace fontlib {
 		// Dumb linear search.
 		for (unsigned int i = 0; i < s_fonts.size(); i++)
 		{
-			font*	f = s_fonts[i].get_ptr();
+			font*	f = s_fonts[i].get();
 			if (f != NULL)
 			{
 				if (strcmp(f->get_name(), name) == 0)
@@ -1305,21 +1309,17 @@ namespace fontlib {
 		// @@ worth it to precompute these bounds?
 
 		rect	bounds = tg.m_uv_bounds;
-		bounds.m_x_min -= tg.m_uv_origin.m_x;
-		bounds.m_x_max -= tg.m_uv_origin.m_x;
-		bounds.m_y_min -= tg.m_uv_origin.m_y;
-		bounds.m_y_max -= tg.m_uv_origin.m_y;
+		bounds.shift_x (-tg.m_uv_origin.m_x);
+		bounds.shift_y (-tg.m_uv_origin.m_y);
 
 		// Scale from uv coords to the 1024x1024 glyph square.
 		// @@ need to factor this out!
 		static float	s_scale = GLYPH_CACHE_TEXTURE_SIZE * s_rendering_box / nominal_glyph_height;
 
-		bounds.m_x_min *= s_scale;
-		bounds.m_x_max *= s_scale;
-		bounds.m_y_min *= s_scale;
-		bounds.m_y_max *= s_scale;
+		bounds.scale_x(s_scale);
+		bounds.scale_y(s_scale);
 		
-		render::draw_bitmap(mat, tg.m_bitmap_info.get_ptr(), bounds, tg.m_uv_bounds, color);
+		render::draw_bitmap(mat, tg.m_bitmap_info.get(), bounds, tg.m_uv_bounds, color);
 	}
 
 
